/*
 * Config.h
 *
 *  Created on: 13/12/2015
 *      Author: Victor
 */
/** @mainpage WIFLY RN-XV LCP1769 LIBRARY
 *
 * @section intro_sec Introduction
 *
 * This library is used to communicate the RN-XV Wifly module with LPC1769 board.
 *
 * @author Victoriano Mora Jimenez
 *
 * @section wifly_sect Wifly driver
 *
 * @subsection wifly_sec Wifly
 * Wifly module with the necessary functions to interact with the RN-XV see @ref Wifly.c.
 *
 * @subsection TCP
 * You can use TCP module to establish TCP communications. see @ref Tcp.c.
 *
 * @subsection UDP
 * You can use UDP module to establish TCP communications. see @ref UDP.c
 *
 * @subsection ARPALAB
 * You can use ARPALAB module to send and receive values from arpalab web. see @ref Arpalab.c
 *
 * @subsection LOG
 * Log module thats you can use to debug your app. see @ref Log.c
 *
 * @subsection Printf
 * Auxiliary module to communicate by an UART port. see @ref PrintF.c
 *
 * @subsection uart
 * Auxiliary module to configure and send or receive characters by an UART port. @ref uart.c
 *
 */

#ifndef CONFIG_H_
#define CONFIG_H_

#include "uart.h"
#include "Printf.h"
#include "WiFly.h"
#include "Log.h"

/** Wifly reset Pin */
#define WIFLY_RST_PIN					22

/* WLAN Settings */
/** WLAN SSID name */
#define WLAN_SSID	 					"MORGUE"

/** WLAN Password Wifly */
#define WLAN_PASSWORD 					"Mireia09&Victor11"

/** WLAN Security type (OPEN, WEP, WPA1,MIXED, WPA2, ADHOCM, WPE64) */
#define WLAN_SEC_TYPE 					MIXED

/** Default WLAN Ping check connection */
#define WLAN_PING_CHECK					"8.8.8.8"

/** Wifly UART port debugger (UART0PORT,UART1PORT,UART2PORT,UART3PORT) */
#define UART_PORT_WIFLY		UART3PORT

/** Size of Wifly buffer */
#define BUFSIZE_WIFLY 		512

/** Debug UART port number (UART0PORT,UART1PORT,UART2PORT,UART3PORT) */
#define UART_PORT_DEBUG		UART0PORT

/** Size of DEBUG buffer */
#define BUFSIZE_DEBUG 		512

/** Debug log level */
#define LOG_LVL 			3

/* Arpalab settings */
/** Arpalab DNS Address */
#define ARPALAB_DNSADDRESS	"arpalab.appspot.com"
/** Arpalab ID Number */
#define ARPALAB_ID 			9423
/** Arpalab Application name */
#define ARPALAB_APP 		"TFC"

/* TCP settings */
/** TCP address connection */
#define TCP_ADDRESS	"192.168.1.34"
/** TCP port connection */
#define TCP_PORT 			6789

/* UDP settings */
/** UDP address connection */
#define UDP_ADDRESS	"192.168.1.34"
/** UDP remote port connection */
#define UDP_PORT 			2589
/** UDP local port connection */
#define UDP_LOCAL_PORT		2589


#endif /* CONFIG_H_ */
