/*
 * Mòdul PrintF que escriu i llegeix cadenes a través de una de les UARTS de
 * la placa LPC1769
 *
 *  Created on: 10/10/2015
 *      Author: Victor
 *  revision 1.0 date 18/09/2015
 */

#ifndef __LOG_H_
#define __LOG_H_


#define STRACE		"TRC->"	/**< Trace prefix */
#define SINFO		"INF->"	/**< Information prefix */
#define SERROR		"ERR->"	/**< Error prefix */
#define SCRLF		"\r\n"	/**< Carriage Return Line Feed */

#define LV_TRACE 3		/**< Trace log level */
#define LV_INFO 2		/**< Info log level */
#define LV_ERR 1		/**< Error log level */
#define LOGSTRING_VALUE "LOG: %s %d\r\n"



/** Initialize UART port, and baud rate transmission.
 *
 * @param PortNum UART port number (0,1,2,3).
 * @param baudrate UART port Baud rate.
 *
 * @return Returns the result of initialization. SUCCESS= TRUE, ERROR = FALSE.
 */
uint32_t Log_start( uint32_t PortNum, uint32_t baudrate );

/** Writes the string format with the selected logLevel. If the severity level of the
 * system chosen is greater than or equal to the level of log message then written
 *
 * @param severity Log level severity of the system.
 *
 * @param logLevel Log level message. (LV_TRACE, LV_INFO, LV_ERROR)
 *
 * @param format String that contains a format string that follows the same specifications as format in printf.
 */
void Log_log(uint8_t severity, uint8_t logLevel, const char * restrict format,...);


/**Writes the string format selected
 *
 * @param format  String that contains a format string that follows the same specifications as format in printf.
 */
void Log_slog(const char * restrict format,...);

/** Writes the selected  log level prefix
 *
 * @param logLevel Log level message.  (LV_TRACE, LV_INFO, LV_ERROR)
 */
void Log_level(uint8_t logLevel);

#endif /* LOG_H_ */
