/*
 * Mòdul PrintF que escriu i llegeix cadenes a través de una de les UARTS de
 * la placa LPC1769
 *
 *  Created on: 10/10/2015
 *      Author: Victor
 *  revision 1.0 date 18/09/2015
 */

#ifndef __PRINTF_H_
#define __PRINTF_H_

#define UART0PORT 0
#define UART1PORT 1
#define UART2PORT 2
#define UART3PORT 3
#define DEFBAUDRATE 9600

/** Initialize UART port, and baud rate transmission.
 *
 * @param portNum UART port number (0,1,2,3).
 * @param baudRate UART port Baud
 *
 * @return Returns the result of initialisation. SUCCESS= TRUE, ERROR = FALSE.
 */
uint32_t PrintF_start( uint32_t portNum, uint32_t baudRate );

/** Close UART port
 *
 * @param PortNum UART port number (0,1,2,3).
 * @return Returns the result of initialisation. SUCCESS= TRUE, ERROR = FALSE.
 */
uint32_t PrintF_stop( uint32_t PortNum);

/** Write printf format string into the selected UART
 *
 * @param portNum UART port number (0,1,2,3).
 *
 * @param format string that contains a format string that follows the same specifications as format in printf.
 *
 */
void PrintF_print( uint32_t portNum, const char * restrict format, ...);

/** Read the response of the UART and moves it to the string parameter
 *
 * @param port UART port number (0,1,2,3).
 *
 * @param string Character string received by the UART
 *
 * @param timeout Waiting time required to receive the answer
 *
 */
void PrintF_scan(uint8_t port, char* string, uint32_t timeout);

/** Llegeix la trama de la UART i la introdueix al paràmetre String
 *
 * @param portNum UART port number (0,1,2,3).
 *
 * @param strScan Character string received by the UART
 *
 * @param strExp Character string that contains expected response.
 *
 * @param timeout Waiting time required to receive the answer
 *
 * @return Returns if the received string is equal to the expected. SUCCESS= TRUE, ERROR = FALSE.
 *
 */
uint8_t PrintF_ChkScan(uint8_t portNum, char *strScan, char *strExp, uint32_t timeout);

/** Initializes the UART port buffer chosen and the PrintF module
 *
 * @param portNum UART port number (0,1,2,3).
 *
 */
void PrintF_ClBuffUart(uint8_t portNum);

/** Time delay
 *
 * @param ms time delay (ms)
 *
 */
void PrintF_delayMs( uint32_t ms);

#endif /* PRINTF_H_ */
