/*
 * Udp.h
 *
 *  Created on: 15/12/2015
 *      Author: Victor Mora
 */

#ifndef UDP_H_
#define UDP_H_

/** Initialise a UDP connection parameters.
 *
 * @param ssid SSID connection name.
 * @param AuthMode Authentication mode number.
 * @param passw Pass phrase of the connection.
 *
 * @return Returns whether the initialisation was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Udp_start(char *ssid, uint32_t AuthMode, char *passw);

/** Send a UDP command and send the server response.
 *
 * @param ipAddress Destination IP address.
 * @param localPort Local port number.
 * @param remotePort Remote port number.
 * @param value Numerical value to send.
 * @return Returns whether the transmission was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Udp_sendValue( char *ipAddress, uint32_t localPort, uint32_t remotePort, uint32_t value);

/** Send a UDP command and receive the server response.
 *
 * @param address Destination IP address.
 * @param localPort Local port number.
 * @param remotePort Remote port number.
 *
 * @return Returns whether the transmission was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
int32_t Udp_receiveValue( char *address, uint32_t localPort, uint32_t remotePort);

#endif /* UDP_H_ */
