/*
 * Log.c
 *
 *  Created on: 10/10/2015
 *      Author: Victor
 */

#include "LPC17xx.h"
#include "stdio.h"
#include "stdlib.h"
#include <string.h>
#include <stdarg.h>
#include "FreeRTOS.h"
#include "semphr.h"

#include "Type.h"
#include "Config.h"
#include "PrintF.h"
#include "Log.h"

/* BUFFER SIZE */
#define LOGBUFSIZE (BUFSIZE_DEBUG)

uint32_t PortNumLog; /**< Stores the UART port number */
char string[LOGBUFSIZE]; /**< Log Buffer to stores the information to send */
xSemaphoreHandle xMutexLOG; /**< Log semaphore Handle */

uint32_t Log_start( uint32_t PortNum, uint32_t baudrate )
{
	uint8_t rtn = FALSE;
	/* Es crea el semàfor si no està creat */
	if( xMutexLOG == NULL )
	{
		xMutexLOG = xSemaphoreCreateMutex();
	}
	/*S'utilitza el semàfor per inicialitzar el Port */
	xSemaphoreTakeRecursive( xMutexLOG, portMAX_DELAY );
	{
	    /*Inicialitza el Port UART a la velocitat indicada */
		PortNumLog=PortNum;
		rtn =PrintF_start(PortNumLog, baudrate);

	}
	xSemaphoreGiveRecursive( xMutexLOG );
	return rtn;
}

void Log_slog(const char * restrict format,...)
{
	va_list arg;
	va_start( arg, format );
	xSemaphoreTakeRecursive( xMutexLOG, portMAX_DELAY );
	{
		/* Mostra el text sel·leccionat amb el Prefix del LOG */
		PrintF_print(PortNumLog, STRACE, format);
	}
	xSemaphoreGiveRecursive( xMutexLOG );
	return;
}

void Log_log( uint8_t severity, uint8_t logLevel, const char * restrict format,...)
{
	va_list arg;

	//Check if the severity level is equivalent to the requested type of log to send
	if (severity>=logLevel)
	{
		xSemaphoreTakeRecursive( xMutexLOG, portMAX_DELAY );
		{
			va_start( arg, format);

			Log_level(logLevel);	/* writes the type of log prefix */
			vsprintf(string, format, arg);
			strcat(string,SCRLF);
			PrintF_print(PortNumLog, string);

			va_end(arg);
		}
		xSemaphoreGiveRecursive( xMutexLOG );
	}
	return;
}

void Log_level(uint8_t logLevel)
{

	xSemaphoreTakeRecursive( xMutexLOG, portMAX_DELAY );
	{
		switch (logLevel)
		{
		case LV_TRACE: /* Trace level */
			PrintF_print(PortNumLog, STRACE);
			break;

		case LV_INFO: /* Info level */
			PrintF_print(PortNumLog, SINFO);
			break;

		case LV_ERR:  /* Error level */
			PrintF_print(PortNumLog, SERROR);
			break;
		}
	}
	xSemaphoreGiveRecursive( xMutexLOG );
	return;
}

