/*
 * PrintF.C
 *
 *  Created on: 10/10/2015
 *      Author: Victor
 */
#include "LPC17xx.h"
#include <stdio.h>
#include "uart.h"
#include "PrintF.h"
#include <string.h>
#include "Type.h"
#include "Config.h"
#include <stdarg.h>

#include "FreeRTOS.h"
#include "semphr.h"

#define BUFSIZE		(BUFSIZE_WIFLY)

/* Es declara la variables de tipus xSemaphoreHandle.*/
xSemaphoreHandle xMutexUART0, xMutexUART1, xMutexUART2, xMutexUART3;
char cUART0Buffer[BUFSIZE], cUART1Buffer[BUFSIZE], cUART2Buffer[BUFSIZE], cUART3Buffer[BUFSIZE];



uint32_t PrintF_start( uint32_t PortNum, uint32_t baudRate )
{
	switch( PortNum )
	{
		case UART0PORT:
			/* Es crea el semàfor si no està creat */
			if( xMutexUART0 == NULL )
			{
				xMutexUART0 = xSemaphoreCreateMutex();
			}
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART0, portMAX_DELAY );
			{
				UARTInit(PortNum, baudRate);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART0 );
			return (TRUE);

		case UART1PORT:
			/* Es crea el semàfor si no està creat */
			if( xMutexUART1 == NULL )
			{
				xMutexUART1 = xSemaphoreCreateMutex();
			}
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART1, portMAX_DELAY );
			{
				UARTInit(PortNum, baudRate);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART1 );
			return (TRUE);

		case UART2PORT:
			/* Es crea el semàfor si no està creat */
			if( xMutexUART2 == NULL )
			{
				xMutexUART2 = xSemaphoreCreateMutex();
			}
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART2, portMAX_DELAY );
			{
				UARTInit(PortNum, baudRate);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART2 );
			return (TRUE);

		case UART3PORT:
			/* Es crea el semàfor si no està creat */
			if( xMutexUART3 == NULL )
			{
				xMutexUART3 = xSemaphoreCreateMutex();
			}
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART3, portMAX_DELAY );
			{
				UARTInit(PortNum, baudRate);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART3 );
			return (TRUE);
	}

	return (FALSE);
}

uint32_t PrintF_stop( uint32_t PortNum)
{
	switch( PortNum )
	{
		case UART0PORT:
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART0, portMAX_DELAY );
			{
				UARTClose(PortNum);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART0 );
			return (TRUE);

		case UART1PORT:
			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART1, portMAX_DELAY );
			{
				UARTClose(PortNum);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART1 );
			return (TRUE);

		case UART2PORT:

			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART2, portMAX_DELAY );
			{
				UARTClose(PortNum);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART2 );
			return (TRUE);

		case UART3PORT:

			/*S'utilitza el semàfor per inicialitzar el Port */
			xSemaphoreTake( xMutexUART3, portMAX_DELAY );
			{
				UARTClose(PortNum);	/* baud rate setting */
			}
			xSemaphoreGive( xMutexUART3 );
			return (TRUE);
	}

	return (FALSE);
}

void PrintF_print( uint32_t PortNum, const char * restrict format, ...)
{
	int lenBuff;
	va_list arg;
	va_start( arg, format );

	PrintF_ClBuffUart(PortNum);

	/* Creació dels semàfors segons la UART a escriure */
	switch( PortNum )
	{
		case UART0PORT:
			/* Es comprova que el semàfor està creat */

			if( xMutexUART0 != NULL )
			{
				lenBuff = vsprintf(cUART0Buffer, format, arg);
				xSemaphoreTake( xMutexUART0, portMAX_DELAY );
				{
					UARTSend(PortNum, (uint8_t *)cUART0Buffer , lenBuff );
				}
			}
			xSemaphoreGive( xMutexUART0 );
			break;

		case UART1PORT:
			if( xMutexUART1 != NULL )
			{
				lenBuff = vsprintf(cUART1Buffer, format, arg);
				xSemaphoreTake( xMutexUART1, portMAX_DELAY );
				{

					UARTSend(PortNum, (uint8_t *)cUART1Buffer, lenBuff);
				}
			}
			xSemaphoreGive( xMutexUART1 );
			break;

		case UART2PORT:
			/* Es comprova que el semàfor està creat */
			if( xMutexUART2 != NULL )
			{
				lenBuff = vsprintf(cUART2Buffer, format, arg);
				xSemaphoreTake( xMutexUART2, portMAX_DELAY );
				{
					UARTSend(PortNum, (uint8_t *)cUART2Buffer , lenBuff );
				}
			}
			xSemaphoreGive( xMutexUART2 );
			break;

		case UART3PORT:
			/* Es comprova que el semàfor està creat */
			if( xMutexUART3 != NULL )
			{
				lenBuff = vsprintf(cUART3Buffer, format, arg);
				xSemaphoreTake( xMutexUART3, portMAX_DELAY );
				{
					UARTSend(PortNum, (uint8_t *)cUART3Buffer , lenBuff );
				}
			}
			xSemaphoreGive( xMutexUART3 );
			break;
	}
	va_end( arg );
	PrintF_ClBuffUart(PortNum);
	return;
}

void PrintF_scan(uint8_t port, char* string, uint32_t timeout)
{
	/* Creació dels semàfors segons la UART a escriure */
	if(port == UART0PORT)
	{
		xSemaphoreTake( xMutexUART0, portMAX_DELAY );
		{
			UARTReceive(port, string, timeout);
		}
		xSemaphoreGive( xMutexUART0 );
	}
	else if(port == UART1PORT)
	{
		xSemaphoreTake( xMutexUART1, portMAX_DELAY );
		{
			UARTReceive(port, string, timeout);
		}
		xSemaphoreGive( xMutexUART1 );
	}
	else if(port == UART2PORT)
	{
		xSemaphoreTake( xMutexUART2, portMAX_DELAY );
		{
			UARTReceive(port, string, timeout);
		}
		xSemaphoreGive( xMutexUART2 );
	}
	else if(port == UART3PORT)
	{
		xSemaphoreTake( xMutexUART3, portMAX_DELAY );
		{
			UARTReceive(port, string, timeout);
		}
		xSemaphoreGive( xMutexUART3 );
	}
   	return;
}

uint8_t PrintF_ChkScan(uint8_t PortNum, char *strScan, char *strExp, uint32_t timeout)
{
	uint8_t rtn = TRUE;

	PrintF_ClBuffUart(PortNum);
	PrintF_scan(PortNum, strScan, timeout);

	if (strstr(strScan, strExp) == NULL)
		rtn = FALSE;
	PrintF_ClBuffUart(PortNum);

	return rtn;
}

void PrintF_ClBuffUart(uint8_t portNum)
{
	clearBuffUart(portNum);
	switch (portNum)
	{
		case UART0PORT:
			memset((uint8_t *)cUART0Buffer, 0x00, BUFSIZE);
			break;
		case UART1PORT:
			memset((uint8_t *)cUART1Buffer, 0x00, BUFSIZE);
			break;
		case UART2PORT:
			memset((uint8_t *)cUART2Buffer, 0x00, BUFSIZE);
			break;
		case UART3PORT:
			memset((uint8_t *)cUART3Buffer, 0x00, BUFSIZE);
			break;
	}
}

void PrintF_delayMs( uint32_t ms)
{
	delayMs(ms,0);
}
