/*
 * Tcp.c
 *
 *  Created on: 25/12/2015
 *      Author: Victor
 */
#include "LPC17xx.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Type.h"

#include "WiFly.h"
#include "Tcp.h"
#include "FreeRTOS.h"
#include "semphr.h"


#define TCP_SEND "SET VALUE=%d\r\n"
#define TCP_RECEIVE "GET RANDOM VALUE\r\n"
#define TCP_SET_RESPONSE "SET OK"
#define TCP_GET_RESPONSE "GET OK"

#define IP_REMOTE 80
#define IP_PROTO 18

xSemaphoreHandle xMutexTcp;
static char *Tcpssid;
static char *TcpPasw;
static uint32_t TcpAuthMode;



/** Start Tcp settings
 *
 * @param ssid SSID name.
 * @param AuthMode Authentication mode:
 * @param passw password
 * @return TRUE, FALSE
 */
uint8_t Tcp_start(char *ssid, uint32_t AuthMode, char *passw)
{
	uint8_t rtn= FALSE;
	if( xMutexTcp == NULL )
	{
		xMutexTcp = xSemaphoreCreateMutex();
	}
	/*S'utilitza el semàfor per inicialitzar el Port */
	xSemaphoreTakeRecursive( xMutexTcp, portMAX_DELAY );
	{
		Tcpssid = ssid;
		TcpAuthMode =AuthMode;
		TcpPasw= passw;
	}
	xSemaphoreGiveRecursive( xMutexTcp );

	return rtn;
}

uint8_t Tcp_sendValue( char *ipAddress, uint32_t remotePort, uint32_t value)
{
	uint8_t rtn= FALSE;
	char commRemote[128];

	xSemaphoreTakeRecursive( xMutexTcp, portMAX_DELAY );
	{
		if(Wifly_WLANConnectPing(TcpAuthMode, Tcpssid, TcpPasw, ipAddress)==TRUE)
		{
			sprintf(commRemote, TCP_SEND,value);
			if(Wifly_TCPSend(ipAddress, remotePort, TCP_SET_RESPONSE, commRemote)==TRUE)
				rtn=TRUE;

		}
	}
	xSemaphoreGiveRecursive( xMutexTcp );

	return rtn;
}

int32_t Tcp_receiveValue( char *address, uint32_t remotePort)
{
	int32_t value;
	char* resp;

	xSemaphoreTakeRecursive( xMutexTcp, portMAX_DELAY );
	{
		if(Wifly_WLANConnectPing(TcpAuthMode, Tcpssid, TcpPasw, address)==TRUE)
		{
			resp= Wifly_TCPReceive(address, remotePort, TCP_RECEIVE);

			resp=strstr(resp, "VALUE=") + strlen("VALUE=");
			value = strtol(resp , &resp, 10);

		}

	}
	xSemaphoreGiveRecursive( xMutexTcp );

	return value;
}
