/*
 * Udp.c
 *
 *  Created on: 25/12/2015
 *      Author: Victor
 */
#include "LPC17xx.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Type.h"

#include "WiFly.h"
#include "Udp.h"
#include "FreeRTOS.h"
#include "semphr.h"


#define UDP_SEND "SET VALUE=%d"
#define UDP_RECEIVE "GET RANDOM VALUE"
#define UDP_SET_RESPONSE "SET OK"
#define UDP_GET_RESPONSE "GET OK"

#define IP_REMOTE 80
#define IP_PROTO 18

xSemaphoreHandle xMutexUdp;
static char *Udpssid;
static char *UdpPasw;
static uint32_t UdpAuthMode;



/** Start Udp settings
 *
 * @param ssid SSID name.
 * @param AuthMode Authentication mode:
 * @param passw password
 * @return TRUE, FALSE
 */
uint8_t Udp_start(char *ssid, uint32_t AuthMode, char *passw)
{
	uint8_t rtn= FALSE;
	if( xMutexUdp == NULL )
	{
		xMutexUdp = xSemaphoreCreateMutex();
	}
	/*S'utilitza el semàfor per inicialitzar el Port */
	xSemaphoreTakeRecursive( xMutexUdp, portMAX_DELAY );
	{
		Udpssid = ssid;
		UdpAuthMode =AuthMode;
		UdpPasw= passw;
	}
	xSemaphoreGiveRecursive( xMutexUdp );

	return rtn;
}

uint8_t Udp_sendValue( char *ipAddress, uint32_t localPort, uint32_t remotePort, uint32_t value)
{
	uint8_t rtn= FALSE;
	char commRemote[128];

	xSemaphoreTakeRecursive( xMutexUdp, portMAX_DELAY );
	{
		if(Wifly_WLANConnectPing(UdpAuthMode, Udpssid, UdpPasw, ipAddress)==TRUE)
		{
			sprintf(commRemote, UDP_SEND,value);
			if(Wifly_UDPSend(ipAddress, localPort, remotePort, UDP_SET_RESPONSE, commRemote)==TRUE)
				rtn=TRUE;

		}
	}
	xSemaphoreGiveRecursive( xMutexUdp );

	return rtn;
}

int32_t Udp_receiveValue( char *address, uint32_t localPort, uint32_t remotePort)
{
	int32_t value;
	char* resp;

	xSemaphoreTakeRecursive( xMutexUdp, portMAX_DELAY );
	{
		if(Wifly_WLANConnectPing(UdpAuthMode, Udpssid, UdpPasw, address)==TRUE)
		{
			resp= Wifly_UDPReceive(address, localPort, remotePort, UDP_RECEIVE);

			resp=strstr(resp, "VALUE=") + strlen("VALUE=");
			value = strtol(resp , &resp, 10);

		}
	}
	xSemaphoreGiveRecursive( xMutexUdp );

	return value;
}
