/*
 * WiFly.c
 *
 *  Created on: 23/10/2015
 *      Author: Victor
 */

#include "LPC17xx.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

#include "Type.h"
#include "PrintF.h"
#include "WiFly.h"
#include "PrintF.h"

#include "Config.h"
#include "FreeRTOS.h"
#include "semphr.h"


/* SET COMMAND String */
#define SET_IP_ADDRESS 		"set ip address %s\r\n"
#define SET_IP_BACKUP 		"set ip backup %s\r\n"
#define SET_IP_DHCP			"set ip dhcp %d\r\n"
#define SET_IP_FLAGS		"set ip flags %x\r\n"
#define SET_IP_GATEWAY		"set ip gateway %s\r\n"
#define SET_IP_HOST			"set ip host %s\r\n"
#define SET_IP_LOCAL_PORT 	"set ip localport %d\r\n"
#define SET_IP_NET			"set ip net %s\r\n"
#define SET_IP_NETMASK		"set ip netmask %s\r\n"
#define SET_IP_PROTO		"set ip proto %d\r\n"
#define SET_IP_REMOTE		"set ip remote %d\r\n"
#define SET_IP_TCP_MODE		"set ip tcp-mode %x\r\n"

#define SET_COM_REM			"set com remote %s\r\n"
#define SET_OP_FORMAT 		"set option format %d\r\n"
#define SET_UART_MD 		"set uart mode %x \r\n"
#define SET_UART_BAUD 		"set uart baud %d \r\n"
#define SET_UART_INST 		"set uart instant %d \r\n"

#define SET_WLAN_TX 		"set wlan tx %d \r\n"

#define SET_WLAN_AUTH 		"set wlan auth %d\r\n"
#define SET_WLAN_KEY 		"set wlan key %d\r\n"
#define SET_WLAN_PHRASE 	"set wlan phrase %s\r\n"
#define SET_WLAN_SSID		"set wlan ssid %s\r\n"
#define SET_WLAN_JOIN		"set wlan join %d\r\n"
#define SET_WLAN_CHAN		"set wlan chan %d\r\n"
#define SET_DNS_NAME 		"set dns name %s\r\n"

/* GET COMMAND String */

#define GET_ADHOC			"get adhoc\r\n"
#define GET_MAC				"get mac\r\n"
#define GET_IP 				"get ip a\r\n"
#define GET_WLAN 			"get wlan\r\n"
#define GET_VER				"ver\r\n"

/* STATUS COMMANDS */
#define SHOW_RSSI 			"show rssi\r\n"
#define SHOW_TIME 			"show time\r\n"
#define SHOW_CONNECTION		 "show connection\r\n"


/* ACTION COMMANDS */
#define AP_MODE				"apmode %s %d"
#define CHG_CMD 			"$$$"
#define CHG_DT 				"exit\r\n"
#define FACTORYRESET 		"factory RESTORE\r\n"
#define CLOSE				"close\r\n"
#define JOIN 				"join %s\r\n"
#define LEAVE 				"leave\r\n"
#define PING 				"ping %s\r\n"
#define OPEN_CONN 			"open %s %d\r\n"
#define REBOOT 				"reboot\r\n"
#define FACTORY_RESET 		"factory RESET\r\n"
#define SLEEP				"sleep\r\n"

/*FILE I/O COMMANDS */
#define SAVE 				"save\r\n"

/* Variables declaration */
xSemaphoreHandle xMutexWifly;
static char wBuffer[WIFLYBUFSIZE];				///< Wifly communication buffer.
static char TcpRxBuff[WIFLYBUFSIZE];			///< TCP response buffer.
static char UdpRxBuff[WIFLYBUFSIZE];			///< UDP response buffer.
static char HTTPRxBuff[WIFLYBUFSIZE];			///< HTTP response buffer.
static uint32_t wiflyPortNum; 					///< Wifly uart port number.
static uint32_t wiflyBaudRate; 					///< Wifly baud rate.
static uint32_t wiflyLogSev= 0;					///< Log severity level to show the different level types.

static char * WiflyIpPing = WLAN_PING_CHECK; 	///< Stores IP direction to use in the Wlan_connect ping check.
static uint8_t WiFlyMode = DATA_M;				///< Wifly mode: DATA_M, CMD_M.
static uint8_t WiflyWlanConn = WLAN_DISCON;		///< WLAN connection status: WLAN_DICON, WLAN_CON, WLAN_APCON.
static uint8_t WiflyProtConfig = MODE_DISCON;	///< Communication protocol mode configured: MODE_DISCON, MODE_UDP, MODE_TCP.
static uint8_t WiflyWLANConnected = FALSE;		///< Wlan connection status.
static uint8_t WiflyTCPAutoConnect = FALSE;		///< TCP auto connection status.
//static uint8_t WiflyUDPAutoConnect = FALSE;		///< UDP auto connection status.

/** Initialise Buffer
 *
 */
static void clearwBuffer()
{
	memset((uint8_t *)wBuffer, 0x00, WIFLYBUFSIZE);
	PrintF_ClBuffUart(wiflyPortNum);
}

/** Send a command and check the response
 *
 * @param delay Response timeout
 * @param exResp Expected response
 * @param format Command string format
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
static uint8_t Wifly_SendSet(uint32_t delay, char *exResp, const char * restrict format,...)
{
	uint8_t rtn = FALSE;
	va_list arg;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		va_start( arg, format);
		clearwBuffer();
		vsprintf(wBuffer, format, arg);

		Log_log( wiflyLogSev, LV_TRACE, "Send command: %s",wBuffer);//Log de la trama enviada
		PrintF_print(wiflyPortNum, wBuffer);
		clearwBuffer();
		rtn = PrintF_ChkScan(wiflyPortNum, wBuffer, exResp, delay);
		//PrintF_delayMs(50);
		Log_log( wiflyLogSev, LV_TRACE, "Response: %s",wBuffer);//Log de la trama enviada
		if (rtn==FALSE)
			Log_log( wiflyLogSev, LV_ERR, "Error expected response: %s",wBuffer);//Log de la trama enviada
		va_end(arg);
	}
	xSemaphoreGiveRecursive( xMutexWifly );

	return rtn;
}

/** Send a command and return the response
 *
 * @param delay Response timeout
 * @param format Command string format
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
static char * Wifly_SendGet(uint32_t delay, const char * restrict format,...)
{
	va_list arg;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		va_start( arg, format);
		clearwBuffer();
		vsprintf(wBuffer, format, arg);
		Log_log( wiflyLogSev, LV_TRACE, "Send command: %s",wBuffer);//Log de la trama enviada
		PrintF_print(wiflyPortNum, wBuffer);
		//string=wBuffer;
		clearwBuffer();
		PrintF_scan(wiflyPortNum,wBuffer,delay);
		Log_log( wiflyLogSev, LV_TRACE, "Response: %s",wBuffer);//Log de la trama enviada
		//string=wBuffer;
		va_end(arg);
	}
	xSemaphoreGiveRecursive( xMutexWifly );

	return wBuffer;
}

/** Send $$$ to change command mode
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
static uint8_t Wifly_CMD()
{
	uint8_t attempts=0;

		while ((WiFlyMode==DATA_M) && (attempts<3))
		{
			if(Wifly_SendSet(CMD_DELAY, CMD, CHG_CMD)==TRUE)//Send $$$ COMMAND
			{
				WiFlyMode=CMD_M;
				return TRUE;
			}
			attempts++;
		}

	if(attempts==3)
	{
		Log_log( wiflyLogSev, LV_ERR, "Error max attempts sending CMD command");
		Wifly_reset(RESET_HARDWARE);
		return FALSE;
	}
	else
		return TRUE;
}

/** Send exit to change command mode
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
static uint8_t Wifly_exitCMD()
{
	if(WiFlyMode==CMD_M)
	{

		if(Wifly_SendSet(CMD_DELAY, EXIT, CHG_DT)==TRUE) //Send exit COMMAND
		WiFlyMode=DATA_M;

	}
	if(WiFlyMode==DATA_M)
		return TRUE;
	else
		return FALSE;
}

uint8_t Wifly_start( uint32_t PortNum, uint32_t baudrate)
{
	uint8_t rtn = FALSE;
	/* Es crea el semàfor si no està creat */
	if( xMutexWifly == NULL )
	{
		xMutexWifly = xSemaphoreCreateMutex();
	}

	/*S'utilitza el semàfor per inicialitzar el Port */
	wiflyPortNum=PortNum;
	wiflyBaudRate=baudrate;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		rtn=PrintF_start(wiflyPortNum, wiflyBaudRate);	/* baud rate setting */
		Wifly_reset(RESET_HARDWARE);
	}
	xSemaphoreGiveRecursive( xMutexWifly );

	if(rtn==FALSE)
		Log_log( wiflyLogSev, LV_ERR, "Error during Wifly UART initialize");
	return rtn;
}

uint8_t Wifly_logStart( uint32_t PortNum, uint32_t baudrate, uint32_t logSeverity )
{
	uint8_t rtn = FALSE;
	/* Es crea el semàfor si no està creat */
	if( xMutexWifly == NULL )
	{
		xMutexWifly = xSemaphoreCreateMutex();
	}
	wiflyLogSev= logSeverity;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		if(wiflyLogSev>0)
		{
			rtn=Log_start(PortNum,baudrate);
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );

	if(rtn==FALSE)
		Log_log( wiflyLogSev, LV_ERR, "Error during Log UART initialize");

	return rtn;
}

uint8_t Wifly_ChangeBaudRate(uint32_t baudrate)
{
	uint8_t rtn=FALSE;
	if(Wifly_setUartBaud(baudrate)==TRUE)
	{
		wiflyBaudRate=baudrate;
		Wifly_save();
		Wifly_reset(RESET_SOFTWARE);
		//PrintF_stop(wiflyPortNum);

	}
	if(PrintF_start(wiflyPortNum,wiflyBaudRate)==TRUE)
	{
		rtn=TRUE;
		PrintF_delayMs(1000);
		clearwBuffer();
	}
	if(rtn==TRUE)
		Log_log( wiflyLogSev, LV_INFO, "Set baud rate: %d", baudrate);
	else
		Log_log( wiflyLogSev, LV_ERR, "Error: Cannot change baud rate");

	return rtn;
}

void Wifly_reset(uint8_t typeReset)
{
	if (typeReset==0)
	{
		// reset wifly device to ensure to start from know state. Connect pin from LPC to reset pin in wifly
		// At this point we will use the same pin to control board led, it's recommend to you another one.
		// Initialise pin
		LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
		LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );
		// it made reset at Wifly
		LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
		PrintF_delayMs(1); // min 160us
		LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;
		Log_log( wiflyLogSev, LV_INFO, "Hardware reset Wifly module");

	}
	else
	{
		if (Wifly_CMD()==TRUE)
		{
			if(Wifly_SendSet(RST_DELAY, READY, REBOOT)==TRUE) //Send Reboot COMMAND
				Log_log( wiflyLogSev, LV_INFO, "Software reset Wifly module");
		}
	}

	//Initialised configured parameters

	WiFlyMode = DATA_M;
	WiflyProtConfig = MODE_DISCON;
	WiflyWlanConn = WLAN_DISCON;

	WiflyWLANConnected=FALSE;

	if( PrintF_ChkScan(wiflyPortNum, wBuffer, READY, RST_DELAY)==TRUE)
	{
		Wifly_CMD();
		Wifly_SendSet(ECHO_DELAY, AOK, SET_UART_MD, UART_MD);//Config ECHO OFF
	}

}

uint8_t Wifly_save()
{
	if(Wifly_SendSet(CMD_DELAY, SAVE_OK, SAVE)==TRUE) //Send exit COMMAND
	{
		return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_factoryReset()
{
	if(Wifly_SendSet(CMD_DELAY, AOK, FACTORY_RESET)==TRUE) //Send exit COMMAND
	{
		return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_sleep()
{
	if(Wifly_SendSet(SET_DELAY, AOK, SLEEP)==TRUE) //Send exit COMMAND
	{
		return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_WLANConnect(uint32_t AuthMode, char *ssid, char* passw)
{
	uint32_t attempts=0;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );

	if(Wifly_WLANConfigConnect(AuthMode, ssid, passw)==TRUE)
	{
		if(WiflyWLANConnected==FALSE)
		{
			if(Wifly_SendSet(JOIN_DELAY,JOIN_OK, JOIN, ssid)==TRUE)//Send join to Associate
			{
				WiflyWLANConnected=TRUE;
			}
		}

		while((Wifly_ping(WiflyIpPing)==FALSE)&&(attempts<3)) //Check wifly connection
		{
			if(Wifly_SendSet(JOIN_DELAY,JOIN_OK, JOIN, ssid)==FALSE)//Send join to Associate
			{
				WiflyWLANConnected=FALSE;
				attempts++;
			}
		}
		if(attempts>=3)//After 3 attempts, will reset module
		{
			Wifly_reset(RESET_HARDWARE);
			xSemaphoreGiveRecursive( xMutexWifly );
			return FALSE;
		}
	}
	else
	{
		xSemaphoreGiveRecursive( xMutexWifly );
		Log_log( wiflyLogSev, LV_ERR, "Error WLAN configuration connection");
		return FALSE;
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return TRUE;
}

uint8_t Wifly_WLANConnectPing(uint32_t AuthMode, char *ssid, char* passw, char* pingIP)
{
	WiflyIpPing=pingIP; /* Change IP address ping to connection  address*/
	return Wifly_WLANConnect(AuthMode, ssid, passw);
}

uint8_t Wifly_leave()
{
	if (Wifly_CMD()==TRUE)
	{
		if(Wifly_SendSet(SET_DELAY, AOK, LEAVE)==TRUE)//Send ping COMMAND
		{
			Log_log( wiflyLogSev, LV_INFO, "Leave WLAN connection");
			return TRUE;
		}
	}
	Log_log( wiflyLogSev, LV_ERR, "Error leave connection");
	return FALSE;
}

uint8_t Wifly_WLANConfigConnect(uint32_t AuthMode, char *ssid, char* passw)
{
		if(Wifly_CMD()==TRUE)
		{
			if(WiflyWlanConn!=WLAN_CON)
			{
				if(Wifly_setWlanJoin(0)==FALSE) //Send WLAN join mode
					return FALSE;
				Wifly_save();
				Wifly_reset(RESET_SOFTWARE);

				if (Wifly_setWlanAuth(AuthMode)==FALSE) //Send WLAN auth mode
					return FALSE;
				if(Wifly_setWlanSSID(ssid)==FALSE) //Send ssid name
					return FALSE;
				if(Wifly_setIpDHCP(1)==FALSE)//set DHCP
					return FALSE;

				if((AuthMode==WPA1)||(AuthMode==WPA2)||(AuthMode==MIXED))
				{
					if(Wifly_setWlanPhrase(passw)==FALSE)//Send WPA password
						return FALSE;
				}
				else if (AuthMode==WEP)
				{
					if(Wifly_setWlanKey(passw)==FALSE)//Send WEP password
						return FALSE;
				}
				WiflyWlanConn=WLAN_CON;
				Log_log( wiflyLogSev, LV_INFO, "WLAN Connection configured");
			}
			return TRUE;
		}
		return FALSE;
}


uint8_t Wifly_ping(char* ip)
{
	if (Wifly_CMD()==TRUE)
	{
		if(Wifly_SendSet(PING_DELAY, PING_OK, PING,ip)==TRUE)//Send ping COMMAND
		{
			Log_log( wiflyLogSev, LV_INFO, "Ping OK: %s",ip);
			return TRUE;
		}
	}
	return FALSE;
}

uint8_t Wifly_open(char* dnsAddress, uint32_t port)
{

	if (Wifly_CMD()==TRUE)
	{
		if(Wifly_SendSet(OPEN_DELAY, OPEN_CON, OPEN_CONN, dnsAddress, port)==TRUE)//Send open COMMAND
		{
			Log_log( wiflyLogSev, LV_INFO, "Open connection:IP= %s, PORT= %d",dnsAddress,port);
			return TRUE;
		}
	}

	return FALSE;
}

uint8_t Wifly_close()
{

	if (Wifly_CMD()==TRUE)
	{
		if(Wifly_SendSet(CMD_DELAY, CLOSE_CON, CLOSE)==TRUE)//Send close COMMAND
		{
			Log_log( wiflyLogSev, LV_INFO, "Close connection");
			return TRUE;
		}
	}
	return FALSE;
}

//SET COMMAND LIST//
uint8_t Wifly_setIpAddress(char * address)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_ADDRESS,address)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpDHCP(uint8_t mode)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_DHCP,mode)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpNET(char * address)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_NET,address)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpFlags(uint32_t mask)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_FLAGS,mask)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpGateway(char* address)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_GATEWAY,address)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpHost(char* address)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_HOST,address)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpLocalPort(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_LOCAL_PORT,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpnetmask(char* address)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_NETMASK,address)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpProtocol(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_PROTO, value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpRemote(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_REMOTE, value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setIpTcpMode(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_IP_TCP_MODE,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setComRemote(char* value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_COM_REM,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setOptionFormat(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_OP_FORMAT,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setUartMode(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_UART_MD,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setUartInstant(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		Wifly_SendSet(SET_DELAY, "", SET_UART_INST,value);//Send COMMAND
		WiFlyMode = DATA_M;
		return TRUE;
	}
	WiFlyMode = DATA_M;
	return FALSE;
}

uint8_t Wifly_setUartBaud(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		Wifly_SendSet(SET_DELAY, AOK, SET_UART_BAUD,value);//Send COMMAND
		//WiFlyMode = DATA_M;
		return TRUE;
	}
	WiFlyMode = DATA_M;
	return FALSE;
}

uint8_t Wifly_setWlanTX(uint32_t tx)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_TX ,tx)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanAuth(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_AUTH ,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanChannel(uint8_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_CHAN ,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanJoin(uint32_t value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_JOIN ,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanKey(char* value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_KEY ,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanPhrase(char* password)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_PHRASE ,password)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setWlanSSID(char* value)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_WLAN_SSID ,value)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_setDNSName(char* name)
{
	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, SET_DNS_NAME ,name)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

char * Wifly_getIp()
{
	char * string;

	if(Wifly_CMD()==TRUE)
	{
		string=Wifly_SendGet(GET_DELAY, GET_IP);;
		string = strtok(string,"\r\n"); //Parse IP address
		Log_log( wiflyLogSev, LV_INFO, "IP= %s",string);
	}

	return string;
}

uint8_t Wifly_getChannel()
{
	uint8_t rtn = FALSE;
	char *string;

	if(Wifly_CMD()==TRUE)
	{
		Wifly_SendGet(GET_DELAY, GET_WLAN);

		string = strstr(wBuffer, "Chan=") + strlen("Chan="); //Parse channel
		rtn = strtol(string , &string, 10);
		Log_log( wiflyLogSev, LV_INFO, "Channel= %d",rtn);
	}

	return rtn;
}

uint8_t Wifly_getTx()
{
	uint8_t tx = 0;
	char *string;

	if(Wifly_CMD()==TRUE)
	{
		Wifly_SendGet(GET_DELAY, GET_WLAN);

		string = strstr(wBuffer, "TxPower=") + strlen("TxPower="); //Parse Tx power
		tx = strtol(string , &string, 10);
		Log_log( wiflyLogSev, LV_INFO, "TxPower= %d",tx);
	}
	return tx;
}

int32_t Wifly_getRSSI()
{
	int32_t rssi=0;
	char *string;

	if(Wifly_CMD()==TRUE)
	{
		Wifly_SendGet(GET_DELAY, SHOW_RSSI);
		string = strstr(wBuffer, "RSSI=(") + strlen("RSSI=("); //Parse RSSI signal
		rssi = strtol(string , &string, 10);
		Log_log( wiflyLogSev, LV_INFO, "RSSI= %d",rssi);
	}
	return rssi;
}

uint32_t Wifly_getTime()
{
	uint32_t time=0;
	char *string;

	if(Wifly_CMD()==TRUE)
	{
		Wifly_SendGet(GET_DELAY, SHOW_TIME);
		string = strstr(wBuffer, "Time=") + strlen("Time="); //Parse Time
		time = strtol(string , &string, 10);
		Log_log( wiflyLogSev, LV_INFO, "Time power up= %d",time);
	}
	return time;
}

char* Wifly_getMac()
{
	char * string;

	if(Wifly_CMD()==TRUE)
	{
		string=Wifly_SendGet(GET_DELAY, GET_MAC);
		string = strstr(wBuffer, "Mac Addr=") + strlen("Mac Addr=");
		string = strtok(string,"\r\n");

		Log_log( wiflyLogSev, LV_INFO, "Mac Addr= %s",string);
	}

	return string;
}

char* Wifly_getWlan()
{
	char * string;

	if(Wifly_CMD()==TRUE)
	{
		string=Wifly_SendGet(GET_DELAY, GET_WLAN);
		string = strstr(wBuffer, "SSID=") + strlen("SSID=");
		string = strtok(string,"\r\n");
		Log_log( wiflyLogSev, LV_INFO, "SSID= %s",string);
	}

	return string;
}

double Wifly_getVer()
{
	double ver;
	char * string;

	if(Wifly_CMD()==TRUE)
	{
		string=Wifly_SendGet(GET_DELAY, GET_VER);
		string = strstr(wBuffer, "Ver: ") + strlen("Ver: ");
		ver = strtod(string , &string);
		Log_log( wiflyLogSev, LV_INFO, "Version= %f",ver);
	}

	return ver;
}

uint32_t Wifly_showConnection()
{
	uint32_t rtn=0;
	char *string;

	if(Wifly_CMD()==FALSE)
	{
		Wifly_SendGet(GET_DELAY, SHOW_CONNECTION);
		string = strstr(wBuffer, "8") + strlen("8");
		rtn = strtol(string , &string, 16);
	}

	return rtn;
}

char * Wifly_HTTPReceive(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto)
{
	char *buff;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		if (Wifly_HTTPConfig(dnsAddress, getCommand, port, proto)==TRUE)
		{
			if(Wifly_open( dnsAddress, port)==TRUE)
			{
				WiFlyMode=DATA_M;
				buff=strcpy(HTTPRxBuff,wBuffer);
				buff = strstr(buff, HTTP_HEAD) + strlen(HTTP_HEAD);//Remove HEADER
				buff = strtok(buff,"*"); //Remove *CLOS*
			}
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return buff;
}

uint8_t Wifly_HTTPSend(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto)
{
	uint8_t rtn=FALSE;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		if (Wifly_HTTPConfig(dnsAddress, getCommand, port, proto)==TRUE)
		{

			if (Wifly_open( dnsAddress, port)==TRUE)
			{
				WiFlyMode=DATA_M;
				rtn=TRUE;
			}
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return rtn;
}

uint8_t Wifly_HTTPConfig(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto)
{

	if(WiflyProtConfig!=MODE_HTTP)
	{

		if(Wifly_setIpProtocol( proto)==FALSE) //set ip proto
			return FALSE;
		if(Wifly_setOptionFormat(1)==FALSE)//set option format
			return FALSE;
		WiflyProtConfig=MODE_HTTP;

	}
	if (Wifly_setDNSName(dnsAddress)==FALSE)// set dns name "dns address"
		return FALSE;
	if(Wifly_setIpRemote(port)==FALSE)//set ip remote
		return FALSE;
	if(Wifly_setComRemote(getCommand)==TRUE)//set com remote
		return TRUE;

	return FALSE;
}

char * Wifly_UDPReceive(char *address, uint32_t localPort, uint32_t remotePort, const char * restrict command,...)
{
	char *buff;
	va_list arg;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		if (Wifly_UDPConfig(address, localPort, remotePort)==TRUE)
		{
			if(Wifly_exitCMD()==TRUE)
			{
				PrintF_delayMs(100);
				va_start( arg, command);
				memset((uint8_t *)UdpRxBuff, 0x00, WIFLYBUFSIZE);
				buff=strcpy(UdpRxBuff,Wifly_SendGet(UDP_DELAY, command, arg));
				va_end(arg);
			}
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return buff;
}

uint8_t Wifly_UDPSend(char *address, uint32_t localPort, uint32_t remotePort, char *exResp , const char * restrict command,...)
{
	uint8_t rtn=FALSE;
	va_list arg;

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
	{
		if (Wifly_UDPConfig(address, localPort, remotePort)==TRUE)
		{
			if(Wifly_exitCMD()==TRUE)
			{
				PrintF_delayMs(100);
				va_start( arg, command);
				if(Wifly_SendSet(UDP_DELAY, exResp, command, arg)==TRUE)
					rtn=TRUE;
				va_end(arg);
			}
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );

	return rtn;
}

uint8_t Wifly_UDPConfig(char *address, uint32_t localPort, uint32_t remotePort)
{

	if (Wifly_CMD()==TRUE)
	{
		if(WiflyProtConfig!=MODE_UDP)
		{
			if(Wifly_setIpProtocol(1)==FALSE) //set ip proto 1, UDP protocol
				return FALSE;
			if(Wifly_setOptionFormat(0)==FALSE)//set option format
				return FALSE;
			WiflyProtConfig=MODE_UDP;
		}
		if (Wifly_setIpHost(address)==FALSE)// set ip host
			return FALSE;
		if(Wifly_setIpRemote(remotePort)==FALSE)//set remote port
			return FALSE;
		if(Wifly_setIpLocalPort(localPort)==FALSE)//set local port
			return FALSE;
		return TRUE;
	}
	return FALSE;
}

char * Wifly_TCPReceive(char *address, uint32_t remotePort, const char * restrict command,...)
{
	char * buff;

	va_list arg;
	va_start( arg, command);


	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );

	if (Wifly_TCPConfig(address,remotePort)==TRUE)
	{
		if(Wifly_open(address,remotePort)==TRUE);
		{

			buff=strcpy(TcpRxBuff,Wifly_SendGet(TCP_DELAY,command, arg));

			va_end(arg);

			WiFlyMode=DATA_M;
			Wifly_close();
		}
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return buff;
}

uint8_t Wifly_TCPSend(char *address, uint32_t remotePort, char *exResp , const char * restrict command,...)
{

	va_list arg;
	va_start( arg, command);

	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );

	if (Wifly_TCPConfig(address, remotePort)==TRUE)
	{
		if(Wifly_open(address,remotePort)==TRUE);
		{


			Wifly_SendSet(TCP_DELAY, exResp, command, arg);
			va_end(arg);

			WiFlyMode=DATA_M;
			Wifly_close();

		}
		xSemaphoreGiveRecursive( xMutexWifly );
		return TRUE;
	}
	xSemaphoreGiveRecursive( xMutexWifly );
	return FALSE;
}
//TODO
void Wifly_TCPTimeOpen(uint32_t time, char *address, uint32_t remotePort, char *exResp , const char * restrict command,...)
{
	char string[128];
	uint8_t attempts=0;
	uint32_t timeSec=time*1000;
	va_list arg;


	while(WiflyTCPAutoConnect==TRUE)
	{
		va_start( arg, command);
		vsprintf(string, command, arg);
		while((Wifly_TCPSend(address, remotePort, exResp , string)==FALSE)&& attempts<3)
		{
			attempts++;
		}
		attempts=0;
		va_end(arg);
		PrintF_delayMs(timeSec);
	}
	return;
}
//TODO
void Wifly_TCPTimeOpenDisc()
{
	WiflyTCPAutoConnect=FALSE;
}

uint8_t Wifly_TCPConfig(char *address, uint32_t port)
{

	if (Wifly_CMD()==TRUE)
	{
		if(WiflyProtConfig!=MODE_TCP)
		{
			if(Wifly_setIpProtocol(2)==FALSE) //set ip proto 2, TCP protocol
				return FALSE;
			if(Wifly_setOptionFormat(0)==FALSE)//set option format
				return FALSE;
			if(Wifly_setComRemote(HELLO_CON)==FALSE)//set com remote
				return FALSE;
			WiflyProtConfig=MODE_TCP;
		}
		if (Wifly_setIpHost(address)==FALSE)// set ip host
			return FALSE;
		if(Wifly_setIpRemote(port)==FALSE)//set remote port
			return FALSE;
		return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_APMode(char *ssid, uint8_t channel)
{

	if(Wifly_CMD()==TRUE) //Check the module is in command mode
	{
		if(Wifly_SendSet(SET_DELAY, AOK, AP_MODE ,ssid, channel)==TRUE)//Send COMMAND
			return TRUE;
	}
	return FALSE;
}

uint8_t Wifly_ConfigAPMode(char *ssid, uint8_t channel,char *ipAddress, char *subnetMask, char *gateway)
{
	double ver;

	ver=Wifly_getVer();
	if(ver>=2.45)
	{
		if (Wifly_CMD()==TRUE)
		{
			xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
			{
				if(WiflyWlanConn!=WLAN_APCON)
				{
					if(Wifly_setWlanJoin(7)==FALSE) //set wlan join
						return FALSE;
					if (Wifly_setWlanChannel(channel)==FALSE)// set wlan channel
						return FALSE;
					if(Wifly_setWlanSSID(ssid)==FALSE)//set wlan ssid
						return FALSE;
					if(Wifly_setIpDHCP(4)==FALSE)//set DHCP
						return FALSE;
					if(Wifly_setIpAddress(ipAddress)==FALSE)//set IP address
						return FALSE;
					if(Wifly_setIpNET(subnetMask)==FALSE)//set IP subnet mask
						return FALSE;
					if(Wifly_setIpGateway(gateway)==FALSE)//set IP gateway
						return FALSE;
					if(Wifly_save()==FALSE)//save settings
						return FALSE;
					Wifly_reset(RESET_SOFTWARE);//reset module
					WiflyWlanConn=WLAN_APCON;
				}
			}
			xSemaphoreGiveRecursive( xMutexWifly );
			return TRUE;
		}
	}
	else
		Log_log( wiflyLogSev, LV_ERR, "Soft AP not compatible with this version:ver= %f",ver);
	return FALSE;
}
