/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo includes. */
#include "basic_io.h"

/* Uart includes */
#include "type.h"
#include "Config.h"

#include "PrintF.h"

#include "Log.h"
#include "WiFly.h"
#include "Arpalab.h"
#include <string.h>

/* Used as a delay. */
#define mainDELAY_LOOP_COUNT 	(20000)
#define mainDELAY_START			(3000)

#define UART_FOR_PRINT UART3PORT
#define UART_FOR_DEBUG UART0PORT
#define BAUD_PRINT DEFBAUDRATE
#define BAUD_DEBUG DEFBAUDRATE

#define PING_IP			(WLAN_PING_CHECK)
#define AUTH_MODE 		(WLAN_SEC_TYPE)
#define SSID			(WLAN_SSID)
#define PASSWORD 		(WLAN_PASSWORD)
#define ARPA_ID 		(ARPALAB_ID)
#define ARPA_APP 		(ARPALAB_APP)
#define DNS_ADDRESS 	(ARPALAB_DNSADDRESS)
#define PRG_TEST 		0 //Selecciona la tasca a realitzar 0 test comandes, !0 test Arpalab

/* The task functions. */
void vTaskStart( void *pvParameters );
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );
void vTask3( void *pvParameters );
void vTask4( void *pvParameters );


/*-----------------------------------------------------------*/

int main( void )
{
	const char* welcomeMsg = "UART3 Online:\r\n";

	SystemInit();	//Called by startup code

	/* Create an Start task */
	xTaskCreate( vTaskStart, "Task Start", 240, NULL, 2, NULL );

	/* Command test */
	if(PRG_TEST==0)
		/* Command test */
		xTaskCreate( vTask1, "Task 1", 240, NULL, 1, NULL );
	else
		/* Arpalab test */
		xTaskCreate( vTask2, "Task 2", 240, NULL, 1, NULL );


	/* Start the scheduler so our tasks start executing. */
	vTaskStartScheduler();
	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/
void vTaskStart( void *pvParameters )
{
	const char *pcTaskName = "Task Start is running";
	Wifly_logStart(UART_FOR_DEBUG, BAUD_DEBUG,LOG_LVL);
	Wifly_start( UART_FOR_PRINT, BAUD_PRINT );
	Arpalab_start( SSID, AUTH_MODE, PASSWORD);


	vTaskDelete(NULL);
	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; );
}
/*-----------------------------------------------------------*/
/*Test commands */
void vTask1( void *pvParameters )
{
	const char *pcTaskName = "Test Get values is running";
	char *string;
	uint8_t channel;
	int32_t rssi;
	uint32_t time;
	uint32_t tx;

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{

		/* Print out the name of this task. */
		Log_log(3,1,(char *)pcTaskName);

		Wifly_ChangeBaudRate(4800);

		Wifly_WLANConnect(AUTH_MODE, SSID, PASSWORD);

		string=Wifly_getIp();
		channel=Wifly_getChannel();
		rssi=Wifly_getRSSI();
		time=Wifly_getTime();

		Wifly_setWlanTX(11);
		tx=Wifly_getTx();

		Wifly_ChangeBaudRate(9600);

		Wifly_getMac();
		Wifly_getWlan();
		Wifly_getVer();

		Wifly_ConfigAPMode("WIFLYAP", 5, "192.168.4.1", "255.255.255.0","192.168.4.1");

		/* Delay for a period */
		vTaskDelay( mainDELAY_LOOP_COUNT);
	}
}
/*-----------------------------------------------------------*/

void vTask2( void *pvParameters )
{
	const char *pcTaskName = "Task 2 is running";
	//char *resp;
	char string[64];
	uint32_t dataD,time, modD;
	int32_t rssi;
	int32_t resValue;
	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{

		/* Print out the name of this task. */
		Log_log(2,2,(char *)pcTaskName);

		dataD=Arpalab_receiveValue(DNS_ADDRESS); //Receive Value

		/* Print out dataD, dataD mod 8 and time out */
		modD=dataD%8; //D mod 8
		time=Wifly_getTime();//Time WiFly POWER UP
		sprintf(string,"D:%d, D mod 8: %d, Time: %d",dataD, modD,time);
		Log_log(3,2,string); //PRINT DATA

		/* Send dataD mod 8 + RSSI to Arpalab server */
		rssi=Wifly_getRSSI();
		resValue=modD+rssi;
		Arpalab_sendValue( DNS_ADDRESS,ARPA_ID, ARPA_APP, resValue);


		/* Delay for a period. */
		vTaskDelay( mainDELAY_LOOP_COUNT);
	}
}


void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	consoleprint("Error there is too little heap RAM\n");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	consoleprint("Error Stack Overflow\n");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


